import java.util.*;


public class LinkedBinaryTree<E> implements BinaryTree<E> {

	private BinaryNode<E> root;
	private int size;


	/**
    * @return an empty binary tree.
    */
    public LinkedBinaryTree( )
    {
    	root =  null;
    	size = 0;
    } // constructor LinkedBinaryTree


    /**
    * @return a binary tree with the specified element as its root.
    */
   	public LinkedBinaryTree (E element) 
   	{
      	size = 1;
      	root = new BinaryNode<E> (element);
   	}  // constructor LinkedBinaryTree



   	/**
    * @return a binary tree from the two specified binary trees.
    */
   	public LinkedBinaryTree (LinkedBinaryTree<E> leftSubtree, E element, LinkedBinaryTree<E> rightSubtree) 
   	{
   		root = new BinaryNode<E> (element);
   		// root.left = null; root.left = null;
   		size = 1;
   		if (! leftSubtree.isEmpty()) {
   			size = size + leftSubtree.size;
   			root.setLeft(leftSubtree.root);
   		} 
   		if (! rightSubtree.isEmpty()) {
   			size = size + rightSubtree.size; 
   			root.setRight(rightSubtree.root);

   		}
	}  // constructor LinkedBinaryTree  	
   	   	

	/**
    * @return the number of elements in the binary tree.
    */
	public int size() 
	{
	      return size;
	}  // method size
	
	
	/**
    * @return the root of the binary tree.
    */
	public BinaryNode<E> getRoot() 
	{
	      return root;
	}  // method root
	
	/**
	* @return a reference to the element if it is found
    * in the root, null otherwise.
    */
 	public E getDataBtree()
 	{
 		return root.getData();
 	}	

   	/**
    * @return true if the binary tree is empty and false otherwise.
    */
    public boolean isEmpty( )
    {
    	return root == null;
	}  // method isEmpty
 

   	/**
    * @return a reference to the specified element if it is found
    * in the binary tree, null otherwise.
    */
   	public E find(E targetElement)  {
     	BinaryNode<E> current = find( targetElement, root );
     	if ( current == null ) return null;
     	else return (current.getData());
   	} // method find



	//================================================================
   	//  Returns a reference to the specified target element if it is
   	//  found in the binary tree.
   	//================================================================
   	private BinaryNode<E> find(E targetElement, BinaryNode<E> root) {
     		if (root == null) return null;
     		if (root.getData().equals(targetElement)) return root;
     		BinaryNode<E> temp = find(targetElement, root.getLeft());
     		if (temp == null) temp = find(targetElement, root.getRight());
     		return temp;
   	} // method find


    /**
     * @return true if the tree contains an element that matches the
     * specified target element and false otherwise.
     */
 	public boolean contains (E targetElement) 
    	{
 		if (find(targetElement, root) != null) return true;
 		else return false;
    	}  // method contains


   	
	/**
     * Deletes all nodes from the binary tree.
     */
 	public void removeAllElements() 
 	{
 		size = 0;
 	    root = null;
 	}  // method removeAllElements
    

 	/**
     * Removes the left subtree of this binary tree.
     */
    public void removeLeftSubtree() 
    {
    	if (root.getLeft() != null)  size = size - root.getLeft().size();
       	root.setLeft(null);
    }  // method removeLeftSubtree


    /**
     * Removes the right subtree of this binary tree.
     */
    public void removeRightSubtree() 
    {
       	if (root.getRight() != null) size = size - root.getRight().size(); 
       	root.setRight(null);
    }  // method removeRightSubtree   	
   	
  		
   	    /**
   	     * @return a new BinaryTree whose root is referenced to the left subtree
   	     */
   	    public BinaryTree<E> getLeftBtree()
   	    { 
   	    	LinkedBinaryTree<E> leftSubtree = new LinkedBinaryTree<E>();
   	    	if (isEmpty() || root.getLeft() == null) return leftSubtree;
   	    	leftSubtree.root = root.getLeft();
   	    	leftSubtree.size = root.getLeft().size(); 
   	    	return leftSubtree;
   	    }
   	 
   	    
   	    /**
   	     * @return a new BinaryTree whose root is referenced to the right subtree
   	     */
   	    public BinaryTree<E> getRightBtree()
   	    {
   	    	LinkedBinaryTree<E> rightSubtree = new LinkedBinaryTree<E>();
   	    	if (isEmpty() || root.getRight() == null) return rightSubtree;
   	    	rightSubtree.root = root.getRight();
   	    	rightSubtree.size = root.getRight().size();
   	    	return rightSubtree;
   	    }
   	    

   	/**
   	 * Performs an inorder traversal on the binary tree by calling
	 * a recursive inorder method that starts with the root.
	 */
	public Iterator<E> iteratorInOrder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	inorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorInOrder


   	/**
   	 * Performs a recursive inorder traversal.
     */
	protected void inorder (BinaryNode<E> node, List<E> templist) 
   	{
	if (node != null) 
 	{
         inorder (node.getLeft(), templist);
         templist.add(node.getData());
         inorder (node.getRight(), templist);
      	}//if

	}  // method inorder



   	/**
	 * Performs a preorder traversal on the binary tree by calling
	 * a recursive preorder method that starts with the root.
     */
	public Iterator<E> iteratorPreOrder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	preorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorPreOrder


   	/**
	 * Performs a recursive preorder traversal.
     */
	protected void preorder (BinaryNode<E> node, List<E> templist) 
   	{
		if (node != null) 
		{
			templist.add(node.getData());
			preorder (node.getLeft(), templist);
			preorder (node.getRight(), templist);
      	}//if
	}  // method preorder



   	/**
   	 * Performs a postorder traversal on the binary tree by calling
   	 * a recursive postorder method that starts with the root.
     */
	public Iterator<E> iteratorPostOrder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	postorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorPostOrder


   	/**
   	 * Performs a recursive postorder traversal.
     */
	protected void postorder (BinaryNode<E> node, List<E> templist) 
   	{
		if (node != null) 
		{
			postorder (node.getLeft(), templist);
			postorder (node.getRight(), templist);
			templist.add(node.getData());
      	}//if
	}  // method postorder 

	
   	/**
   	 * Performs a levelorder traversal on the binary tree using a templist
     */
	public Iterator<E> iteratorLevelOrder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	levelorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorLevelOrder


   	/**
   	 * Performs an iterative levelorder traversal.
     */
   	protected void levelorder(BinaryNode<E> node, List<E> templist) 
   	{
   		LinkedQueue<BinaryNode<E>> queueOfnodes = new LinkedQueue<BinaryNode<E>>();  
      	BinaryNode<E> current;

      	queueOfnodes.add (node);

      	while (! queueOfnodes.isEmpty()) 
      	{
         current = queueOfnodes.remove();
         templist.add(current.getData());
         if (current.getLeft() != null)  queueOfnodes.add (current.getLeft());
         if (current.getRight() != null) queueOfnodes.add (current.getRight());
      	}//while
        }  // method levelorder 



   	/**
   	 * @return a string representation of the binary tree.
     */
   	public String toString() 
   	{
      	List<E> templist = new ArrayList<E>();
      	preorder (root, templist);

	// test:
      	//itpreorder (root, templist);
      	//itinorder (root, templist);

      	return templist.toString();
   	}  // method toString


	/**
	 * @return true if the binary trees have the same structure and
 	 * contain the same values, false otherwise. 
	 */
	public boolean equals(Object other) {
		LinkedBinaryTree<?> t2;
		if(!(other instanceof LinkedBinaryTree<?>))
			return false;
		else t2 = (LinkedBinaryTree<?>) other;
		if (this.isEmpty() && t2.isEmpty()) return true;
		if (this.isEmpty() != t2.isEmpty() ) return false;
		return ( root.equals(t2.root) );
	}



    
   	/**
	  * Performs an iterative preorder traversal on the binary tree using a templist
      */

	public Iterator<E> iteratorItPreorder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	itpreorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorItPreorder


   	/**
	  * Performs an iterative preorder traversal.
      */
   	protected void itpreorder(BinaryNode<E> node, List<E> templist) 
   	{

   		LinkedStack<BinaryNode<E>> stackOfNodes = new LinkedStack<BinaryNode<E>>();  // stack of nodes
      	BinaryNode<E> current;

      	stackOfNodes.push(node);

      	while (! stackOfNodes.isEmpty()) 
	{
         current = stackOfNodes.pop();
         templist.add(current.getData());
         if (current.getRight() != null) stackOfNodes.push (current.getRight());
         if (current.getLeft() != null)  stackOfNodes.push(current.getLeft());
      	}//while
        }  // method itpreorder 


   	/**
	  * Performs an iterative inorder traversal on the binary tree using a templist
	  */

	public Iterator<E> iteratorItInorder() 
   	{
      	ArrayList<E> templist = new ArrayList<E>();
      	itinorder (root, templist);
      	return templist.iterator();
   	}  // method iteratorItInorder


   	/**
	  * Performs an iterative inorder traversal.
      */
   	protected void itinorder(BinaryNode<E> node, List<E> templist) 
   	{
   		LinkedStack<BinaryNode<E>> stackOfNodes = new LinkedStack<BinaryNode<E>>();
   		LinkedStack<Boolean> flags = new LinkedStack<Boolean>();  // stack of boolean values

      	BinaryNode<E> current;
      	boolean visit;

      	stackOfNodes.push(node);
      	flags.push(false);

      	while (!stackOfNodes.isEmpty()) 
      	{
      		current = stackOfNodes.pop();
      		visit = flags.pop();
      		if (visit) templist.add(current.getData());
      		else {
      			if (current.getRight() != null) { stackOfNodes.push(current.getRight()); flags.push(false); }
      			stackOfNodes.push(current); flags.push(true);
      			if (current.getLeft() != null)  { stackOfNodes.push(current.getLeft());  flags.push(false);}
      		} //end-if
      	}//end-while
       
   	}  // method itinorder 


/***************** HOMEWORK *******************************************
		public Iterator<E> iteratorItPostOrder() 
***********************************************************************/
}
